﻿//**********************************************************************
//  DLMS/COSEM components
//
//  i-cube, 1376 Goumoens-la-Ville, Switzerland
//          infos@icube.ch
//
// (C)Copyright 2004-2017
//**********************************************************************

using System;
using System.Collections.Generic;
using System.Text;

using System.Threading;
using System.IO;
using System.Net;
using System.Net.Sockets;
namespace ICube.HdlcDemo
{
    using ICube.Wrapper;
    using ICube.XmlPdu;
    using ICube.XmlPdu.Infra;

    /// <summary>
    /// Base class for Hdlc demo.
    /// </summary>
    class WrapperDemo
    {
        private TcpClient tcpClient_;
        protected ICube.Wrapper.WRAPPERParameters wrapperParameters;
        protected ICube.Wrapper.WRAPPERProtocol wrapper = null;
        protected DateTime dateTime_ = DateTime.Now;

        /// <summary>
        /// Initializes parametres structure.
        /// <remarks>Adapt to your meter parameters.</remarks>
        /// <param name="filename"></param>
        /// </summary>
        protected void InitializeParameters()
        {
            this.wrapperParameters.ClientAddress = 0x10;
            this.wrapperParameters.ServerAddress = 1;
        }

        /// <summary>
        /// Processes an xml file.
        /// </summary>
        /// <param name="filePath"></param>
        protected virtual bool ProcessXmlFile(string filePath)
        {
            bool _success = false;
            StringBuilder _outXml, _inXml;
            MemoryStream _outPdu, _inPdu;

            Console.WriteLine("--------        --------        --------        --------        --------");
            Console.WriteLine("Processing: " + filePath);

            try
            {
                _outXml = new StringBuilder(File.ReadAllText(filePath));
                Console.Write(_outXml.ToString());
                Console.WriteLine();
                _outPdu = XmlPduInterface.XmlToPdu(_outXml);

                _inPdu = this.wrapper.SendReceive(_outPdu.ToArray());
                _inXml = XmlPduInterface.PduToXml(_inPdu);

                Console.Write(_inXml.ToString());
                _success = true;
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
            Console.WriteLine();
            return _success;
        }

        public void doIt(string[] args)
        {
            Console.WriteLine("WRAPPER Demo");
            Console.WriteLine(ICube.EzHdlc.Version.VersionStr);
            Console.WriteLine(ICube.XmlPdu.Version.VersionStr);
            Console.WriteLine(ICube.Wrapper.Version.VersionStr);

            this.InitializeParameters();

            // create TCP/IP connection
            this.tcpClient_ = new TcpClient("127.0.0.1", 4058);
            //this.tcpClient_ = new TcpClient("178.192.6.51", 4001);
            IPEndPoint _ep = (IPEndPoint)tcpClient_.Client.RemoteEndPoint;
            String ID = string.Format("TCP:{0}:{1}", _ep.Address, _ep.Port);
            this.tcpClient_.Client.ReceiveTimeout = 3000;

            // Create Wrapper
            this.wrapper = new WRAPPERProtocol(this.wrapperParameters, this.tcpClient_.GetStream());
            {
                this.ProcessXmlFile("..\\..\\xml-files\\" + "AssociationRequest_LN_NoPW.xml");
                this.ProcessXmlFile("..\\..\\xml-files\\" + "GetRequestNormal(Clock).xml");
            }
        }

        static void Main(string[] args)
        {
            try
            {

                WrapperDemo test = new WrapperDemo();
                test.doIt(args);

            }
            catch (WRAPPERException e)
            {
                Console.WriteLine("WRAPPER exception: {0}", e.Message);

            }
            catch (XmlPduException e)
            {
                Console.WriteLine("XmlPduException exception: {0}", e.Message);

            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);

            }
            Console.WriteLine("Done hit any key !!!");
            Console.ReadKey();
        }



        protected void OnSentStream(object stream)
        {
            if (stream.GetType() == typeof(byte[]))
                this.LogLine("CB_S: " + ICube.EzHdlc.Utils.BytesToHexString((byte[])stream));
            else
                this.LogLine("CB_S: " + (string)stream);
        }

        protected void OnReceivedStream(object stream)
        {
            if (stream.GetType() == typeof(byte[]))
                this.LogLine("CB_R: " + ICube.EzHdlc.Utils.BytesToHexString((byte[])stream));
            else
                this.LogLine("CB_R: " + (string)stream);
        }

        protected void LogLine(string log)
        {
            TimeSpan _timeSpan = DateTime.Now - this.dateTime_;
            string _timeTag = _timeSpan.TotalSeconds.ToString("F3");
            Console.WriteLine("{0,9} : {1}", _timeTag, log);
        }

    }
}
